@@ -1,9 +1,11 @@
 package org.github.jamm;
 
+import java.nio.ByteBuffer;
+
 /**
  * Listener that receive notification form MemoryMeter.
  */
-interface MemoryMeterListener {
+public interface MemoryMeterListener {
 
     /**
      * A factory for <code>MemoryMeterListener</code>.
@@ -28,6 +30,15 @@ interface MemoryMeterListener {
      */
     void fieldAdded(Object obj, String fieldName, Object fieldValue);
 
+    /**
+     * Notification that the element from the specified array has been added.
+     *
+     * @param array the array for which an element has been added
+     * @param index the element index
+     * @param elementValue the element value
+     */
+    void arrayElementAdded(Object[] array, int index, Object elementValue);
+
     /**
      * Notification that the size of the specified object has been measured.
      *
@@ -37,15 +48,25 @@ interface MemoryMeterListener {
     void objectMeasured(Object current, long size);
 
     /**
-     * Notification that the specified object been counted.
+     * Notification that the size of the remaining bytes of a {@code ByteBuffer} have been measured.
      *
-     * @param current the object that has been counted
+     * @param buffer the {@code ByteBuffer}
+     * @param size the remaining bytes
      */
-    void objectCounted(Object current);
+    void byteBufferRemainingMeasured(ByteBuffer buffer, long size);
 
     /**
-     * Notification that the entire graphs has been measured.
+     * Notification that the entire graph has been measured.
      * @param size the size of the entire graph.
      */
     void done(long size);
+
+    /**
+     * Notification that the graph could not be fully measured has it failed to access a field.
+     *
+     * @param obj the object owning the field that could not be accessed
+     * @param fieldName the field name
+     * @param fieldType the field type
+     */
+    void failedToAccessField(Object obj, String fieldName, Class<?> fieldType);
 }
