@@ -1,6 +1,8 @@
 package org.github.jamm;
 
 import org.junit.*;
+import org.junit.runner.RunWith;
+import org.junit.runners.Parameterized;
 
 import java.io.BufferedInputStream;
 import java.io.BufferedReader;
@@ -14,10 +16,9 @@ import java.io.InputStreamReader;
 import java.lang.reflect.Array;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.HashSet;
+import java.util.Collection;
 import java.util.List;
 import java.util.Random;
-import java.util.Set;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ConcurrentSkipListMap;
 import java.util.concurrent.ExecutionException;
@@ -34,12 +35,36 @@ import java.util.regex.Pattern;
  * We generate java source code for random data classes (with parent hierarchy), write it to a temp dir,
  * call javac on it, and instrument it.
  */
+@RunWith(Parameterized.class)
 public class GuessTest {
 
+    @BeforeClass
+    public static void logInfoAtStartup() {
+        System.setProperty("org.github.jamm.strategies.LogInfoAtStartup", "true");
+    }
+
+    @Parameterized.Parameters
+    public static Collection<MemoryMeter.Guess> guesses() {
+
+        List<MemoryMeter.Guess> guesses = new ArrayList<>();
+
+        guesses.add(MemoryMeter.Guess.INSTRUMENTATION_AND_SPECIFICATION);
+        guesses.add(MemoryMeter.Guess.UNSAFE);
+        guesses.add(MemoryMeter.Guess.SPECIFICATION);
+        return guesses;
+    }
+
+    private final MemoryMeter.Guess guess;
+    
+    public GuessTest(MemoryMeter.Guess guess)
+    {
+        this.guess = guess;
+    }
+
     @Test
-    public void testDeepNecessaryClasses() {
-        final MemoryMeter instrument = new MemoryMeter().withTrackerProvider(TRACKER_PROVIDER);
-        final MemoryMeter guess = new MemoryMeter().withGuessing(MemoryMeter.Guess.ALWAYS_SPEC).withTrackerProvider(TRACKER_PROVIDER);
+    public void testDeepClasses() {
+        final MemoryMeter instrument = MemoryMeter.builder().build();
+        final MemoryMeter meter = MemoryMeter.builder().withGuessing(guess).build();
         Assert.assertTrue("MemoryMeter not initialised", MemoryMeter.hasInstrumentation());
         final List<Object> objects = new ArrayList<Object>(); {
             final ConcurrentSkipListMap<Long, Long> map = new ConcurrentSkipListMap<Long, Long>();
@@ -50,34 +75,9 @@ public class GuessTest {
         int failures = 0;
         for (final Object obj : objects) {
             long instrumented = instrument.measureDeep(obj);
-            long guessed = guess.measureDeep(obj);
+            long guessed = meter.measureDeep(obj);
             if (instrumented != guessed) {
-                System.err.println(String.format("Guessed %d, instrumented %d for %s", guessed, instrumented, obj.getClass().getName()));
-                failures++;
-            }
-        }
-        Assert.assertEquals("Not all guesses matched the instrumented values. See output for details.", 0, failures);
-    }
-
-    @Test
-    public void testProblemClasses() throws InterruptedException, ExecutionException, IOException, IllegalAccessException, InstantiationException {
-        final MemoryMeter instrument = new MemoryMeter();
-        final MemoryMeter guess = new MemoryMeter().withGuessing(MemoryMeter.Guess.ALWAYS_UNSAFE);
-        Assert.assertTrue("MemoryMeter not initialised", MemoryMeter.hasInstrumentation());
-        List<Def> defs = new ArrayList<Def>();
-        defs.add(Def.parse("{long*1}->{float*1}"));
-        defs.add(Def.parse("{long*1}->{byte*4}"));
-        defs.add(Def.parse("{long*1}->{byte*7}"));
-        defs.add(Def.parse("{long*1}->{byte*9}"));
-        defs.add(Def.parse("{long*1}->{float*1}->{long*1}->{float*1}"));
-        final List<GeneratedClass> classes = compile(defs);
-        int failures = 0;
-        for (final GeneratedClass clazz : classes) {
-            Object obj = clazz.clazz.newInstance();
-            long instrumented = instrument.measure(obj);
-            long guessed = guess.measure(obj);
-            if (instrumented != guessed) {
-                System.err.println(String.format("Guessed %d, instrumented %d for %s", guessed, instrumented, clazz.description));
+                System.err.printf("Guessed %d, instrumented %d for %s and Guess %s%n", guessed, instrumented, obj.getClass().getName(), guess);
                 failures++;
             }
         }
@@ -87,8 +87,8 @@ public class GuessTest {
     @Test
     public void testRandomClasses() throws InterruptedException, ExecutionException {
         final int testsPerCPU = 100;
-        final MemoryMeter instrument = new MemoryMeter();
-        final MemoryMeter guess = new MemoryMeter().withGuessing(MemoryMeter.Guess.ALWAYS_UNSAFE);
+        final MemoryMeter instrument = MemoryMeter.builder().build();
+        final MemoryMeter meter = MemoryMeter.builder().withGuessing(guess).build();
         Assert.assertTrue("MemoryMeter not initialised", MemoryMeter.hasInstrumentation());
         final List<Future<Integer>> results = new ArrayList<Future<Integer>>();
         for (int i = 0 ; i < Runtime.getRuntime().availableProcessors() ; i++) {
@@ -100,9 +100,9 @@ public class GuessTest {
                     for (final GeneratedClass clazz : classes) {
                         Object obj = clazz.clazz.newInstance();
                         long instrumented = instrument.measure(obj);
-                        long guessed = guess.measure(obj);
+                        long guessed = meter.measure(obj);
                         if (instrumented != guessed) {
-                            System.err.println(String.format("Guessed %d, instrumented %d for %s", guessed, instrumented, clazz.description));
+                            System.err.println(String.format("Guessed %d, instrumented %d for %s and Guess %s", guessed, instrumented, clazz.description, guess));
                             failures++;
                         }
                     }
@@ -118,8 +118,8 @@ public class GuessTest {
 
     @Test
     public void testRandomArrays() throws InterruptedException, ExecutionException {
-        final MemoryMeter instrument = new MemoryMeter();
-        final MemoryMeter guess = new MemoryMeter().withGuessing(MemoryMeter.Guess.ALWAYS_UNSAFE);
+        final MemoryMeter instrument = MemoryMeter.builder().build();
+        final MemoryMeter meter = MemoryMeter.builder().withGuessing(guess).build();
         Assert.assertTrue("MemoryMeter not initialised", MemoryMeter.hasInstrumentation());
         final List<Future<Boolean>> results = new ArrayList<Future<Boolean>>();
         for (int i = 0 ; i < 10000 ; i++) {
@@ -128,9 +128,9 @@ public class GuessTest {
                 public Boolean call() throws Exception {
                     Object obj = Array.newInstance(TYPES[rnd.nextInt(TYPES.length)].clazz, rnd.nextInt(1000));
                     long instrumented = instrument.measure(obj);
-                    long guessed = guess.measure(obj);
+                    long guessed = meter.measure(obj);
                     if (instrumented != guessed) {
-                        System.err.println(String.format("%s of length %d. Guessed %d, instrumented %d", obj.getClass(), Array.getLength(obj), guessed, instrumented));
+                        System.err.println(String.format("%s of length %d. Guessed %d, instrumented %d and Guess %s", obj.getClass(), Array.getLength(obj), guessed, instrumented, guess));
                         return Boolean.FALSE;
                     }
                     return Boolean.TRUE;
@@ -141,13 +141,43 @@ public class GuessTest {
             Assert.assertTrue("Failed test - see output for details", result.get());
     }
 
+    /**
+     * Test method that can be used to reproduce test failures found by the testRandomXXX methods
+     */
+//  @Test 
+    public void testProblemClasses() throws Exception {
+        final MemoryMeter instrument = MemoryMeter.builder().build();
+        final MemoryMeter meter = MemoryMeter.builder().withGuessing(guess).build();
+        Assert.assertTrue("MemoryMeter not initialised", MemoryMeter.hasInstrumentation());
+        List<Def> defs = new ArrayList<Def>();
+        defs.add(Def.parse("{int*1 char*2}->{boolean*1 float*2}"));
+        defs.add(Def.parse("{double*1 byte*1}->{double*1 boolean*5}")); 
+        defs.add(Def.parse("{}->{int*1 double*2}"));
+        defs.add(Def.parse("{double*1}->{int*2 boolean*1}->{byte*3 short*1 double*1}"));
+        defs.add(Def.parse("{float*2}->{long*1 boolean*2 char*1}"));
+        defs.add(Def.parse("{byte*3}"));
+        defs.add(Def.parse("{char*3 long*1}->{short*1 int*2}"));
+
+        final List<GeneratedClass> classes = compile(defs);
+        int failures = 0;
+        for (final GeneratedClass clazz : classes) {
+            Object obj = clazz.clazz.newInstance();
+            long instrumented = instrument.measure(obj);
+            long guessed = meter.measure(obj);
+            if (instrumented != guessed) {
+                System.err.printf("Guessed %d, instrumented %d for %s%n", guessed, instrumented, clazz.description);
+                failures++;
+            }
+        }
+        Assert.assertEquals("Not all guesses matched the instrumented values. See output for details.", 0, failures);
+    }
+
     private static final Types[] TYPES = Types.values();
     private static final Random rnd = new Random();
     private static final AtomicInteger id = new AtomicInteger();
     private static final MyClassLoader CL = new MyClassLoader();
     private static final File tempDir = new File(System.getProperty("java.io.tmpdir"), "testclasses");
 
-    private static final Callable<Set<Object>> TRACKER_PROVIDER = new TrackerProvider();
     private static final ExecutorService CONSUME_PROCESS_OUTPUT = Executors.newCachedThreadPool(new DaemonThreadFactory());
     private static final ExecutorService EXEC = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
 
@@ -171,6 +201,12 @@ public class GuessTest {
             this.clazz = clazz;
             this.name = clazz.getSimpleName();
         }
+
+        @Override
+        public String toString()
+        {
+            return clazz.getSimpleName();
+        }
     }
 
     // a class loader for loading our random classes; permits loading arbitrary bytes to arbitrary class name
@@ -238,7 +274,7 @@ public class GuessTest {
 
         // parse one of our readable descriptions into a Def
         private static Def parse(String description) {
-            final Pattern clazz = Pattern.compile("\\{([a-zO]+\\*[0-9]+ ?)+\\}");
+            final Pattern clazz = Pattern.compile("\\{([a-zO]+\\*[0-9]+ ?)*\\}");
             final Pattern type = Pattern.compile("([a-zO]+)\\*([0-9]+)");
             Matcher cm = clazz.matcher(description);
             List<ClassDef> classdefs = new ArrayList<ClassDef>();
@@ -282,9 +318,9 @@ public class GuessTest {
                 File src = new File(tempDir, name + ".java");
                 if (src.exists())
                     throw new IllegalStateException();
-                final FileWriter writer = new FileWriter(src);
-                writer.append(decl);
-                writer.close();
+                try (FileWriter writer = new FileWriter(src)) {
+                    writer.append(decl);
+                }
                 File trg = new File(tempDir, name + ".class");
                 parts.add(new Decl.ClassDecl(src, trg, name, decl.toString()));
                 prev = name;
@@ -417,13 +453,6 @@ public class GuessTest {
         }
     }
 
-    private static final class TrackerProvider implements Callable<Set<Object>> {
-        @Override
-        public Set<Object> call() throws Exception {
-            return new HashSet<Object>();
-        }
-    };
-
     private static final class DaemonThreadFactory implements ThreadFactory {
         @Override
         public Thread newThread(Runnable r) {
